/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/

#include <stdio.h>
#include "main.h"
#include"usercode.h"


/* Exported functions ---------------------------------------------------------*/
extern void InitializeArrayData(void);
/* Global Variables ---------------------------------------------------------*/
xQueueHandle xQueueCAN2,xQueueCAN1,xQueueSendArray;
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/*--------------- Tasks Priority -------------*/

#define TASK_PRIO    ( tskIDLE_PRIORITY + 1 )

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/



/* Private function prototypes -----------------------------------------------*/
static void LED_Init(void);
void CRC_Module_Init(void);
/* Private functions ---------------------------------------------------------*/

/**
* @brief  Polls den Taster, wenn der Taster bet�tigt wurde wird die Task zum senden des Array gestartet
* @param  pvParameters not used
* @retval None
*/
void ButtonApli(void * pvParameters)
{
	uint8_t buttonstate=0, old_button=0;

	for(;;)
	 {
		/*Check if Button is pressed*/
		if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0) && old_button == 0)
		{
			/*Put the Button state into the Queue to unblock it*/
			xQueueSendToFront( xQueueSendArray, &buttonstate, 0 );

		}

		old_button=GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0);

		vTaskDelay(200);
	}
}



/**
  * @brief  Resets the red LED
  * @param  pvParameters not used
  * @retval None
  */
void LED(void * pvParameters)
{

      for( ;; )
      {

    	vTaskDelay(100);

    	GPIO_ToggleBits(GPIOD, GPIO_Pin_15);

      }

}

/**
  * @brief  Main program anfangs werden alle Peripherien initialisiert
  * 		danach werden die Tasks erzeugt.
  * @param  None
  * @retval None
  */
int main(void)
{
	portBASE_TYPE buttonApplication = 0;
	portBASE_TYPE arraySendApplication = 0;
	portBASE_TYPE ledApplication = 0;
	int32_t checkTasks = 0;
	int32_t numTasks = 3;
	int32_t pauseValue;

	SystemInit();

	/*!< At this stage the microcontroller clock setting is already configured to
	   168 MHz, this is done through SystemInit() function which is called from
	   startup file (startup_stm32f4xx.s) before to branch to application main.
	   To reconfigure the default setting of SystemInit() function, refer to
	   system_stm32f4xx.c file
	 */

	/*Initialize Leds */
	LED_Init();
	/*Initialize CRC Module*/
	CRC_Module_Init();
	/*Configure the Interrupt*/
	InterruptCAN_Init();

	CAN_Initialisierung();

	InitializeArrayData();

	/*Create queue*/
	xQueueSendArray = xQueueCreate( 1, sizeof( uint8_t ) );

	/*Queue successful created*/
	if( xQueueSendArray == 0 )
	{
		while(1)
		;
	}

	/* Create Tasks */
	buttonApplication=xTaskCreate( ButtonApli, NULL, configMINIMAL_STACK_SIZE, NULL, TASK_PRIO, NULL);
	arraySendApplication=xTaskCreate( ArraySend, NULL, 1024, NULL, TASK_PRIO, NULL);
	ledApplication=xTaskCreate( LED, NULL, configMINIMAL_STACK_SIZE, NULL, TASK_PRIO, NULL);

	checkTasks = (int32_t)buttonApplication + (int32_t)arraySendApplication + (int32_t)ledApplication;

	if(checkTasks != numTasks)
	{
		GPIO_SetBits(GPIOD, GPIO_Pin_15);
		GPIO_SetBits(GPIOD, GPIO_Pin_14);
		GPIO_SetBits(GPIOD, GPIO_Pin_13);
		GPIO_SetBits(GPIOD, GPIO_Pin_12);
		while(1)
		{
			for(pauseValue=0; pauseValue<1000000; pauseValue++);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_15);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_14);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_13);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_12);
		}
	}

	/* Start scheduler */
	vTaskStartScheduler();

	/* We should never get here as control is now taken by the scheduler */
	for( ;; );
}



/**
  * @brief  Initializes the LEDs resources.
  * @param  None
  * @retval None
  */
void LED_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

  /* Initialize Discovery's LEDs */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

  /* Configure PD15 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12|GPIO_Pin_13|  GPIO_Pin_14|  GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

  /* Configure PA0 input pulldown mode*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;

  GPIO_Init(GPIOA, &GPIO_InitStructure);

}

void CRC_Module_Init(void)
{
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_CRC, ENABLE);
}




/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
