#include"usercode.h"
#include "stm32f4xx.h"
#include "stm32f4xx_crc.h"
#include "stm32f4xx_can.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_exti.h"
#include "stm32f4xx_syscfg.h"
#include "stm32f4xx_rcc.h"
#include "misc.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "interrupt.h"
#include "stm32f4xx_tim.h"

#define SWITCH(X) (X
#define CHECK(X) X)
#define CRC_TEST SWITCH (0)
#define CHECKSUM_TEST SWITCH (1)

#if CHECK(CRC_TEST)
uint32_t Calc_CRC(uint8_t *data, uint16_t arrayLength);
uint8_t Check_REC_CRC(uint32_t fu, uint32_t bar);
uint32_t checkCRC = 0;
#endif

#if CHECK(CHECKSUM_TEST)
uint32_t checkSum = 0;
uint32_t calcCheckSum(uint8_t *data, uint16_t arrayLength);
#endif

extern xQueueHandle xQueueSendArray;
extern  uint8_t CAN_Send(CAN_TypeDef *CAN,CanTxMsg *Message);
static uint8_t array_send[2000];


/**
  * @brief  Task zum Senden des Arrays
  * @param  pvParameters not used
  * @retval None
  */
void ArraySend(void * pvParameters)
{
	CanTxMsg Message;
	static uint8_t buttonstate=0;
	uint8_t transmitstatus=0;


	for( ;; )
	{
		/*Prepare the Start Message*/
		Message.IDE	= CAN_ID_STD;
		Message.RTR = CAN_RTR_DATA;

		/*Wait till the Queue gets unblocked*/
		xQueueReceive( xQueueSendArray, &( buttonstate), portMAX_DELAY );

		/* toggle LED */
		GPIO_ToggleBits(GPIOD, GPIO_Pin_13);
		// ---------------------------------------------------------------------



		// Startmessage
		Message.StdId	=		0x100;
		Message.DLC		=		0;

		//Code hier eingeben

		//ende der Eingabe

		vTaskDelay(1);
		// Datenmessage
		Message.StdId	=		0x101;

		//Code hier eintragen


		//bitte bis hier Code eintragen

		vTaskDelay(5);
		// Endmessage
		Message.StdId=0x102;
		Message.DLC=4;
		#if CHECK(CRC_TEST)
		Message.Data[0] = ((checkCRC&0xFF000000)>>24);
		Message.Data[1] = ((checkCRC&0xFF0000)>>16);
		Message.Data[2] = ((checkCRC&0xFF00)>>8);
		Message.Data[3] = (checkCRC&0xFF);
		#endif

		#if CHECK(CHECKSUM_TEST)
		Message.Data[0] = ((checkSum&0xFF000000)>>24);
		Message.Data[1] = ((checkSum&0xFF0000)>>16);
		Message.Data[2] = ((checkSum&0xFF00)>>8);
		Message.Data[3] = (checkSum&0xFF);
		#endif

		//Code hier eingeben

		//ende der Eingabe

		/* toggle LED */
		GPIO_ToggleBits(GPIOD, GPIO_Pin_13);
	}

}

void InitializeArrayData(void)
{
	uint16_t arrayIndex;
	for(arrayIndex=0;arrayIndex<2000;arrayIndex++)
	{
		array_send[arrayIndex]=0x00;
		if((arrayIndex%10)==0)
		{
			array_send[arrayIndex]=0x01;
		}
	}
	#if CHECK(CRC_TEST)
	checkCRC=Calc_CRC(array_send,2000);
	#endif

	#if CHECK(CHECKSUM_TEST)
	checkSum=calcCheckSum(array_send, 2000);
	#endif


}

#if CHECK(CRC_TEST)
/**
  * @brief  Calculates the CRC
  * @param  start pointer for the 7 bytes
  * @retval the CRC value
  */
uint32_t Calc_CRC(uint8_t *data, uint16_t arrayLength)
{
	uint32_t CRC_Check=0;
	uint32_t CRC_Proceed=0;
	uint32_t* CRC_pBuff;
	CRC_ResetDR();
	if((arrayLength%4)==0)
	{
		CRC_pBuff = (uint32_t*)data;
		CRC_Proceed=arrayLength>>2;
		CRC_Check=CRC_CalcBlockCRC(CRC_pBuff, CRC_Proceed);
	}
	else
	{
		while(1)
		{
			GPIO_SetBits(GPIOD, GPIO_Pin_14);
		}
	}
	return (CRC_Check);
}
/**
  * @brief  calculates the CRC (used for checking of received frames)
  * @param  the data to check
  * @retval the CRC value (0 if the received CRC is correct)
  */
uint8_t Check_REC_CRC(uint32_t fu, uint32_t bar)
{
	uint8_t isEqual=0;
	isEqual=(fu == bar);
	return(isEqual);
}
#endif

#if CHECK(CHECKSUM_TEST)
uint32_t calcCheckSum(uint8_t *data, uint16_t arrayLength)
{
	uint32_t checkSum=0;
	uint16_t index=0;
	for(index=0;index<arrayLength;index++)
	{
		checkSum+=data[index];
	}
	return(checkSum);
}
#endif
